JSONLogic evaluation engine for processing schemas.
This module provides the core engine that evaluates JSONLogic expressions
with the custom fake operator for generating fake data.
Structs
- Engine
- The core evaluation engine for JSONLogic expressions with fake data support.